# -*- coding:utf-8 -*-
'''
@Project     : fypy

@File        : version.py

@Modify Time :  2022/11/11 14:09   

@Author      : Lee    

@Version     : 1.0   

@Description :

'''

import json

version_json = '''
{
 "version": "1.0.0"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)

# file generated by setuptools_scm
# don't change, don't track in version control
# version = '1.1.13'fff
# version_tuple = (1, 1, 13)